package sample03;

import java.util.function.IntBinaryOperator;
import java.util.function.IntUnaryOperator;

public class HigherOrderFunctionSample {

    public static void main(String[] args) {

        //足し算を行う関数
        IntBinaryOperator add = (a, b) -> a + b;
        int ret1 = add.applyAsInt(1, 2);
        System.out.println(ret1); // 3

        //Java 7までの書き方
        IntBinaryOperator java7Add = new IntBinaryOperator() {
            @Override
            public int applyAsInt(int a, int b) {
                return a + b;
            }
        };
        int ret2 = java7Add.applyAsInt(2, 3);
        System.out.println(ret2); // 5

        //引数に関数を渡す
        int ret3 = calculate(add, 3, 4);
        System.out.println(ret3); //7

        //異なる関数を渡すことも可能
        int ret4 = calculate((a, b) -> a * b, 3, 4);
        System.out.println(ret4); //12

        //戻り値の関数を実行する
        IntUnaryOperator addPartial = addPartial(5);
        int ret5 = addPartial.applyAsInt(6);
        System.out.println(ret5); //11
    }

    //引数に関数を受け取る高階メソッドの例
    static int calculate(IntBinaryOperator fn, int a, int b) {
        return fn.applyAsInt(a, b);
    }

    //戻り値として関数を返す高階メソッドの例
    static IntUnaryOperator addPartial(int a) {
        return b -> a + b;
    }
}
