package sample05;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.io.UncheckedIOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collections;
import java.util.List;

public class StreamAndIOSample {

    public void useStream() {

        BufferedWriter out = new BufferedWriter(new StringWriter());
        List<Path> files = Collections.emptyList();
        files.stream().filter(file -> {
            //1000バイト以上のファイルに絞り込む
            try {
                return Files.size(file) >= 1000L;
            } catch (IOException e) {
                throw new UncheckedIOException(e);
            }
        }).map(file -> {
            //ファイルの1行目を読み込む
            try (BufferedReader in = Files.newBufferedReader(file)) {
                return in.readLine();
            } catch (IOException e) {
                throw new UncheckedIOException(e);
            }
        }).forEach(line -> {
            //別ファイルへ書き出す
            try {
                out.write(line);
                out.newLine();
            } catch (IOException e) {
                throw new UncheckedIOException(e);
            }
        });
    }

    public void notUseStream() throws IOException {

        BufferedWriter out = new BufferedWriter(new StringWriter());
        List<Path> files = Collections.emptyList();
        for (Path file : files) {
            if (Files.size(file) >= 1000L) {
                try (BufferedReader in = Files.newBufferedReader(file)) {
                    out.write(in.readLine());
                    out.newLine();
                }
            }
        }
    }
}
